/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: SwingUtil.java,v 1.4.2.2 2000/04/01 00:14:28 pmahar Exp $
 */

package org.enhydra.devsys.swing;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.UIManager;

/**
 * SwingUtil contains static utility methods for working with Swing
 * classes.
 *
 * @author
 * Paul Mahar
 *
 * @version %I%, %G%
 */
public class SwingUtil {
    private static Frame owner; // JDK 1.1

    public static void setDialogOwner(Frame f) {  // JDK 1.1
      owner = f;                                  // JDK 1.1
    }                                             // JDK 1.1

//    public static Dialog getDialogOwner(Dialog dialog) { // Java 2
//      return dialog;                                     // Java 2
    public static Frame getDialogOwner(Dialog dialog) {    // JDK 1.1
      return owner;                                        // JDK 1.1
    }

    /**
     * Open a JFileChooser dialog for selecting a directory and return the
     * selected directory.
     *
     *
     * @param owner
     * The frame or dialog that controls the invokation of this dialog.
     * @param defaultDir
     * A string representation of the directory to show when the
     * dialog opens.
     * @param title
     * Tile for the dialog.
     *
     * @return
     * The selected directory as a File. Null if user cancels dialog without
     * a selection.
     *
     */
    public static File getDirectoryChoice(Component owner, String defaultDir,
					  String title) {
	return getDirectoryChoice(owner, new File(defaultDir), title);
    }

    /**
     * Open a JFileChooser dialog for selecting a directory and return the
     * selected directory.
     *
     *
     * @param owner
     * The frame or dialog that controls the invokation of this dialog.
     * @param defaultDir
     * The directory to show when the dialog opens.
     * @param title
     * Tile for the dialog.
     *
     * @return
     * The selected directory as a File. Null if user cancels dialog without
     * a selection.
     *
     */
    public static File getDirectoryChoice(Component owner, File defaultDir,
					  String title) {
	File	     choice = null;
	JFileChooser chooser = new JFileChooser();

	chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
	chooser.setFileFilter((javax.swing.filechooser.FileFilter) new DirectoryFilter());
        if ((defaultDir != null) && defaultDir.exists() && defaultDir.isDirectory()) {
  	  chooser.setCurrentDirectory(defaultDir);
	  chooser.setSelectedFile(defaultDir);
        }
	chooser.setDialogTitle(title);
	chooser.setApproveButtonText("OK");

	int v = chooser.showOpenDialog(owner);

	owner.requestFocus();

	if (v == JFileChooser.APPROVE_OPTION
		&& chooser.getSelectedFile() != null) {
	    if (chooser.getSelectedFile().exists()) {
		choice = chooser.getSelectedFile();
	    } else {
                File parentFile = new File(chooser.getSelectedFile().getParent());
		choice = parentFile;
	    }
	}

	chooser.removeAll();

	chooser = null;

	return choice;
    }

    /**
     * Get the point on point on the screen at which to open a dialog
     * or window for it to appear centered. This point is the top right hand
     * corner of the container you want to position.
     *
     *
     * @param size
     * The demensions of the dialog or window to position.
     *
     * @return
     * The top left hand point at which to position the container
     * for it to appear centered.
     *
     */
    public static Point getCenteringPoint(Dimension size) {
	Point     centeringPoint = new Point();
	Dimension screenSize;

	screenSize = Toolkit.getDefaultToolkit().getScreenSize();

	if (size.height > screenSize.height) {
	    size.height = screenSize.height;
	}

	if (size.width > screenSize.width) {
	    size.width = screenSize.width;
	}

	centeringPoint.x = (screenSize.width - size.width) / 2;
	centeringPoint.y = (screenSize.height - size.height) / 2;

	return centeringPoint;
    }

    /**
     * Sets the look and feel to the native operating system look and feel.
     * This method ignores any exceptions that may occur when you set
     * the look and feel.
     *
     */
    public static void setLookAndFeelToSystem() {
	try {
	    String systemLook = UIManager.getSystemLookAndFeelClassName();

	    UIManager.setLookAndFeel(systemLook);
	} catch (javax.swing.UnsupportedLookAndFeelException e) {}
	catch (IllegalAccessException e) {}
	catch (InstantiationException e) {}
	catch (ClassNotFoundException e) {}
	;
    }

}
