/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCBuildThread.java,v 1.1.2.9 2000/04/06 02:53:40 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard.xmlc;

import org.enhydra.ide.jbuilder.common.wizard.AbstractBuildThread;
import org.enhydra.ide.jbuilder.common.FileUtil;
import org.enhydra.ide.jbuilder.common.node.Mapper;
import org.enhydra.ide.jbuilder.common.node.OtterImageFileNode;
import org.enhydra.ide.jbuilder.common.node.OtterProject;
import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.ide.jbuilder.common.capture.Writer;
import org.enhydra.ide.jbuilder.common.capture.WriteListener;
import org.enhydra.xml.xmlc.XMLCException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.File;

/**
 * CompileThread calls XMLC once for each selected HTML file.
 *
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
public class XMLCBuildThread extends AbstractBuildThread {
    /**
     * Create a CompileThread with listeners ready to
     * recieve output from XMLC.
     *
     *
     * @param listeners
     * A listener implements an onWrite method that is called
     * whenever XMLC writes output. The writes are captured
     * for display in a wizard.
     */
    public XMLCBuildThread(WriteListener[] listeners) {
      this(listeners, false);
    }

    public XMLCBuildThread(WriteListener[] listeners,
                         boolean buffer) {
        super(listeners, buffer);
    }

    /**
     * Loop through all selected HTML files and call XMLC for each
     * and write out exceptions to any registered listeners.
     */
    public void make() {
      if (getProject().isMapImages()) {
        makeImages();
      }
      if (getNodes().length > 0) {
        makeSelected();
      }

    }

    private void makeImages() {
      // Find all images in project and copy using
      // the mapping table.
      OtterImageFileNode[] images = getProject().getAllImages();
      Mapper mapper = new Mapper(getProject());
      File source = null;
      File dest   = null;
      for (int i = 0 ; i < images.length ; i++ ) {
         source = new File(images[i].getFilename());
         dest   = new File(mapper.getOutputImagePath(images[i]));
         try {
           FileUtil.copyFile(source, dest);
         } catch (java.io.IOException e) {
           e.printStackTrace();
         }
      }
    }

    private void makeSelected() {
	int	   count = getNodes().length;
	XMLCRunner runner = new XMLCRunner();

	runner.setTraceWriter(getTraceWriter());

	for (int i = 0; i < count; i++) {
            OtterXMLCNode xmlcNode = (OtterXMLCNode) getNodes()[i];
            xmlcNode.setException(null);
	    if (!isFresh()) {
		break;
	    }

	    try {
		if (xmlcNode.isSelected() && isFresh()) {
                    File sourceDoc = xmlcNode.getOptions().getSourceDocument();
                    if (sourceDoc.exists()){
                       if (isPrintProgress()) {
     		          getTraceWriter().println("Compiling: ");
		          getTraceWriter().println("  "
                            + sourceDoc.getAbsolutePath());
                       }
                       xmlcNode.preCompile();
  		       runner.setNode(xmlcNode);
		       runner.compile();
                    } else {
                      if (isPrintProgress()) {
                        getTraceWriter().println("Source document not found: " +
                          sourceDoc.getAbsolutePath());
                      }
                    }
		}
	    } catch (XMLCException e) {
		xmlcNode.setException(e);
                if (isPrintProgress()) {
  		  getTraceWriter().println("");
                }

	    } catch (org.xml.sax.SAXException e) {
		 xmlcNode.setException(e);
                if (isPrintProgress()) {
  		  getTraceWriter().println("");
                 }

	    } catch (java.io.IOException e) {
       	        xmlcNode.setException(e);
                if (isPrintProgress()) {
  		  getTraceWriter().println("");
                }
	    }
	}

	if (isFresh()) {
          if (isPrintProgress()) {
            printResults();
          }
          printExceptions();
          if (isPrintProgress()) {
            getTraceWriter().println("");
	    getTraceWriter().println("--- XMLC finished compiling ---");
	    getTraceWriter().println("");
          }
	} else {
           printExceptions();
	   getTraceWriter().println("");
	   getTraceWriter().println("--- XMLC interrupted ---");
     	   getTraceWriter().println("");
	}

    }

    /**
     * Determine what files were created and write out the
     * compilation results to any registered listeners.
     */
    public void printResults() {
	int eCount = 0;
        int totalCount = getNodes().length;
        int selectCount = 0;

	getTraceWriter().println("");
	getTraceWriter().println("Gathering results...");

	for (int i = 0; i < totalCount; i++) {
	    if (getNodes()[i].isSelected() &&
                getNodes()[i].getException() != null) {
		eCount++;
	    }
	}

	sleepInSeconds(0.5);

	for (int i = 0; i < totalCount; i++) {
	  if (getNodes()[i].isSelected()) {
	    selectCount++;
          }
        }


	if (eCount < selectCount) {
	    getTraceWriter().println("");
	    getTraceWriter().println("New java files:");

	    for (int i = 0; i < totalCount; i++) {
                OtterXMLCNode xmlcNode = (OtterXMLCNode) getNodes()[i];
		sleepInSeconds(0.5);

		if (!isFresh()) {
		    break;
		}

		if (xmlcNode.isSelected()
  		  && xmlcNode.isNewJavaFound()) {
		    xmlcNode.replaceGeneratedSource();
		    getTraceWriter().println(" "
					+ xmlcNode.getOptions().getJavaClassSource());
		}
	    }
	}

	sleepInSeconds(0.5);

      }
}
