/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: XMLCRunner.java,v 1.16.2.4 2000/04/10 05:58:37 pmahar Exp $
 */

package org.enhydra.ide.jbuilder.common.wizard.xmlc;

import org.enhydra.ide.jbuilder.common.node.OtterXMLCNode;
import org.enhydra.xml.xmlc.html.HTMLProprietaryTags;
import org.enhydra.xml.xmlc.commands.xmlc.*;
import org.enhydra.xml.xmlc.XMLCUtil;
import org.enhydra.xml.xmlc.XMLCVersion;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.compiler.*;
import org.enhydra.xml.xmlc.commands.options.*;
import org.enhydra.xml.xmlc.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.io.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Class derived from the XMLC compiler program for use in this wizard.
 * TODO: Mark may change some things that will require changes to this class.
 */
public class XMLCRunner {

    /*
     * Object containing the parsed standard options.
     */
    private OtterXMLCNode   node;

    /*
     * Document being compiled.
     */
    private Document      document;

    /*
     * Error output.
     */
    private ErrorReporter errorReporter;

    /*
     * Trace and verbose output.
     */
    private PrintWriter   traceWriter = new PrintWriter(System.err, true);

    /*
     * XML DOM management object.
     */
    private XMLCRunnerDoc runnerDoc;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCRunner() {
      try {
        runnerDoc = new XMLCRunnerDoc();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }

    /**
     * Method declaration
     *
     *
     * @param tw
     *
     * @see
     */
    public void setTraceWriter(PrintWriter tw) {
	traceWriter = tw;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public PrintWriter getTraceWriter() {
	return traceWriter;
    }

    /*
     * Print out a stack trace.  This works around bugs in SAXException that
     * cause printing a stack trace to generate a null pointer exception.  We
     * can't just fix this in the OpenXML source (we did), because other
     * parsers include that same broken class.
     */

    /**
     * Method declaration
     *
     *
     * @param except
     *
     * @see
     */
    private void printStackTrace(Throwable except) {

	// FIXME: needed???
	if (except instanceof SAXException) {
	    Exception except2 = ((SAXException) except).getException();

	    if (except2 != null) {
		except2.printStackTrace();

		return;
	    }
	}

	except.printStackTrace();
    }

    /*
     * Handle an exception, printing the approriate error
     * message.
     */

    /**
     * Method declaration
     *
     *
     * @param except
     *
     * @see
     */
    public void printException(Throwable except) {

	// Must check that options have been parsed before using them.
	if (except instanceof XMLCException) {
	    System.err.println("Error: " + except.getMessage());

	    if ((node.getOptions() != null) && (node.getOptions().getVerbose())) {
		printStackTrace(except);
	    }

	    // If this is result of a java.lang exception, print stack.
	    Throwable cause = ((XMLCException) except).getCause();

	    if ((cause != null)
		    && cause.getClass().getName().startsWith("java.lang.")) {
		printStackTrace(cause);
	    }
	} else if ((except instanceof FileNotFoundException)
		   || (except instanceof IOException)) {
	    System.err.println("Error: " + except.getClass().getName() + ": "
			       + except.getMessage());

	    if ((node.getOptions() != null) && (node.getOptions().getVerbose())) {
		printStackTrace(except);
	    }
	} else {
	    System.err.println("Error: " + except.getClass().getName() + ": "
			       + except.getMessage());
	    printStackTrace(except);
	}

	System.exit(1);
    }

    /*
     * Parse the page into the DOM and perform various checks and edits.
     */

    /**
     * Method declaration
     *
     *
     * @throws FileNotFoundException
     * @throws IOException
     * @throws SAXException
     * @throws XMLCException
     *
     * @see
     */




    /*
     * Parse the page into the DOM and perform various checks and edits.
     */
    private void parsePage()
        throws XMLCException, FileNotFoundException, IOException {
        if (node.getOptions().getVerbose()) {
            traceWriter.println(">>> parsing " + node.getOptions().getSourceDocument());
        }

        Parse parser = new Parse(errorReporter,
                                (node.getOptions().getParseInfo() ? traceWriter : null));
        Object xmlcDoc = callParse(parser, node.getOptions());
        runnerDoc.setXMLCDoc(xmlcDoc);
        document = runnerDoc.getDocument();

        if (node.getOptions().getPrintInfo()) {
            if (node.getOptions().getVerbose()) {
                traceWriter.println(">>> printing document info");
            }
            LocalPrintInfo info = new LocalPrintInfo(document, runnerDoc);
            info.printInfo(traceWriter);
        }


        EditDOM domEditor = new EditDOM();
        domEditor.addURLEdits(node.getOptions().getURLEdits());
        domEditor.addDeleteClasses(node.getOptions().getDeleteElementClasses());
        callEdit(domEditor, runnerDoc);

        if (node.getOptions().getDump()) {
            if (node.getOptions().getVerbose()) {
                traceWriter.println(">>> dumping document hierarchy");
            }
            XMLCUtil.printNode("DOM hierarchy", document,
                               XMLCUtil.PRINT_ALL,
                               traceWriter);
        }
    }


    /*
     * Generate the Java source files.
     */
    private void generateJavaSource() throws XMLCException, IOException {
        if (node.getOptions().getVerbose()) {
            traceWriter.println(">>> generating code");
        }
        CodeGenerator codeGen = createNewCodeGenerator(
                             node.getOptions(),
                             runnerDoc,
                             (node.getOptions().getPrintAccessMethods() ? traceWriter : null));
        callGenerateJavaSource(codeGen, node.getOptions());
    }

    /*
     * Generate method information without generating code.
     */
    private void generateJavaSourceInfo() throws XMLCException, IOException {
        if (node.getOptions().getVerbose()) {
            traceWriter.println(">>> generating code info without generating source");
        }
        CodeGenerator codeGen = createNewCodeGenerator(
                      node.getOptions(),
                      runnerDoc,
                      (node.getOptions().getPrintAccessMethods() ? traceWriter : null));
    }


    /*
     * Compile the generate source.
     */
    private void compileJavaSource() throws XMLCException {
        if (node.getOptions().getVerbose()) {
            traceWriter.println(">>> compiling code");
        }
        Javac javac = new Javac(node.getOptions().getJavac());
        javac.compile(node.getOptions(),
                      errorReporter,
                      node.getOptions().getVerbose() ? traceWriter : null);
    }

    /**
     * Write the DOM to a file.
    **/
    private void writeDOM() throws IOException {
        String dir = node.getOptions().getDocumentOutput().getParent();
        if (dir != null) {
            new File(dir).mkdirs();
        }
        PrintWriter domOut = new PrintWriter
            (new BufferedOutputStream(new FileOutputStream(node.getOptions().getDocumentOutput())));
        try {
            domOut.println(runnerDoc.toDocument());
        } finally {
            domOut.close();
        }
    }


    /*
     * Generate the Jave source and Compile the page.
     */
    private void compilePage()
        throws XMLCException, FileNotFoundException, IOException {
        try {
            generateJavaSource();
            // do not compile Java
        } finally {
            // never delete source.
            if (!node.getOptions().getKeep()) {
                if (node.getOptions().getJavaClassSource() != null) {
                    node.getOptions().getJavaClassSource().delete();
                }
                if (node.getOptions().getJavaInterfaceSource() != null) {
                    node.getOptions().getJavaInterfaceSource().delete();
                }
            }
        }
    }

    /**
     * Print the XMLC version number.
     */
    private void printVersion() {
        System.out.println("Enhydra XMLC version " + XMLCVersion.VERSION);
        System.out.println("See http://www.enhydra.org for latest distribution");
    }

    /*
     * Parse arguments and compile the page.
     */

    /**
     * Method declaration
     *
     *
     * @throws FileNotFoundException
     * @throws IOException
     * @throws SAXException
     * @throws XMLCException
     *
     * @see
     */
    public void compile()
	    throws XMLCException, FileNotFoundException, IOException,
		   SAXException {

	// Setup and document parsing.
        node.initProjectOptions();
	if (node.getOptions().getPrintVersion()) {
	    printVersion();

	    if (node.getOptions().getSourceDocument() == null) {
		return;
	    }
	}
        errorReporter = createNewErrorReporter(traceWriter,
	      node.getOptions().getSourceDocument().getPath());
	parsePage();

	if (node.getOptions().getDocumentOutput() == null) {
	    compilePage();
	} else {
	    if (node.getOptions().getPrintAccessMethods()) {
		generateJavaSourceInfo();
	    }

	    writeDOM();
	}

	if (node.getOptions().getVerbose()) {
	    traceWriter.println(">>> completed");
	}
    }

    /**
     * Method declaration
     *
     *
     * @param cn
     *
     * @see
     */
    public void setNode(OtterXMLCNode n) {
	node = n;
    }

    private Object callParse(Parse parser, CompileParams cp)
         throws XMLCException {
      Object doc = null;
      Method meths[] = parser.getClass().getDeclaredMethods();
      Method parse = null;
      for (int i = 0 ; i < meths.length ; i++) {
        if (meths[i].getName().equals("parse")) {
          parse = meths[i];
          break;
        }
      }
      if (parse != null) {
        Object params[] = new Object[1];
        params[0] = cp;
        try {
          doc = parse.invoke(parser, params);
        } catch (java.lang.IllegalAccessException e) {
          e.printStackTrace();
        } catch (java.lang.reflect.InvocationTargetException e) {
          if (e.getTargetException() instanceof XMLCException) {
            throw (XMLCException) e.getTargetException();
          } else {
            XMLCException xe = new XMLCException(e.getTargetException().toString());
            e.getTargetException().printStackTrace();
            throw xe;
          }
        }
      }
      return doc;
    }

    private void callGenerateJavaSource(CodeGenerator codeGen, CompileParams compParams) {
      //   codeGen.generateJavaSource(node.getOptions(), null);
      Method[] meths      = codeGen.getClass().getMethods();
      Method   generate   = null;
      Class[]  paramTypes = null;
      for (int i = 0 ; i < meths.length ; i++) {
        if (meths[i].getName().equals("generateJavaSource")) {
          generate = meths[i];
          break;
        }
      }
      if (generate != null) {
        paramTypes = generate.getParameterTypes();
        Object paramValues[] = new Object[paramTypes.length];
        paramValues[0] = compParams;
        if (paramValues.length == 2) {
          paramValues[1] = traceWriter;
        }
        try {
          generate.invoke(codeGen, paramValues);
        } catch (Exception e) {
          e.printStackTrace();
        }
      }

    }



    private void callEdit(EditDOM domEditor, XMLCRunnerDoc runnerDoc) {
      Method meths[] = domEditor.getClass().getDeclaredMethods();
      Method edit = null;
      for (int i = 0 ; i < meths.length ; i++) {
        if (meths[i].getName().equals("edit")) {
          edit = meths[i];
          break;
        }
      }
      if (edit != null) {
        Object params[] = new Object[1];
        params[0] = runnerDoc.getXMLCDoc();
        try {
          edit.invoke(domEditor, params);
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    }



   // This method allows us to create a new instance using the
   // 2.3 or 3.0 XMLC API.
   private ErrorReporter createNewErrorReporter(PrintWriter writer,
                                                String docPath ) {
      ErrorReporter reporter = null;
      try {
        Class repClass = Class.forName("org.enhydra.xml.xmlc.compiler.ErrorReporter");
        Constructor[] cons        = repClass.getConstructors();
        Constructor   constructor = null;
        Class[]       params      = null;
        for (int i = 0 ; i < cons.length ; i++ ) {
          params = cons[i].getParameterTypes();
          // stop on non-default constructors.
          if (params.length > 0) {
            constructor = cons[i];
            break;
          }
        }
        Object[] values = new Object[0];
        if (params.length == 1) {
          values = new Object[1];
          values[0] = writer;
        } else {
          values = new Object[2];
          values[0] = writer;
          values[1] = docPath;
        }
        reporter = (ErrorReporter) constructor.newInstance(values);
      } catch (Exception e) {
        reporter = null;
        e.printStackTrace();
      }
      return reporter;
      //	    new ErrorReporter(traceWriter);  // 3.0
      //	    new ErrorReporter(traceWriter,  // 2.3
      //			      node.getOptions().getSourceDocument().getPath());
   }

   // This method allows us to create a new instance using the
   // 2.3 or 3.0 XMLC API.
   private CodeGenerator createNewCodeGenerator(CompileParams compileParams,
                                                XMLCRunnerDoc runnerDoc,
                                                PrintWriter   methodOutput )
         throws XMLCException {
      CodeGenerator generator = null;
      try {
        Class repClass = Class.forName("org.enhydra.xml.xmlc.compiler.CodeGenerator");
        Constructor[] cons        = repClass.getConstructors();
        Constructor   constructor = null;
        Class[]       params      = null;
        for (int i = 0 ; i < cons.length ; i++ ) {
          params = cons[i].getParameterTypes();
          // stop on non-default constructors.
          if (params.length == 3) {
            constructor = cons[i];
            break;
          }
        }
        Object[] values = new Object[3];
        values[0] = compileParams;
        values[1] = runnerDoc.getXMLCDoc();
        values[2] = methodOutput;
        generator = (CodeGenerator) constructor.newInstance(values);
      } catch (java.lang.reflect.InvocationTargetException e) {
        generator = null;
        if (e.getTargetException() instanceof XMLCException) {
          throw (XMLCException) e.getTargetException();
        } else {
          XMLCException xe = new XMLCException(e.getTargetException().toString());
          e.getTargetException().printStackTrace();
          throw xe;
        }
      } catch (Exception e) {
        generator = null;
        e.printStackTrace();
      }
      return generator;
      //	    new ErrorReporter(traceWriter);  // 3.0
      //	    new ErrorReporter(traceWriter,  // 2.3
      //			      node.getOptions().getSourceDocument().getPath());

   }



  private class LocalPrintInfo {
    /**
     * Table URLs.
     */
    private Vector urls = new Vector();


    /**
     * Table of element ids.
     */
    private Vector ids = new Vector();

    /**
     * Construct document info from document.
     */
    public LocalPrintInfo(Document doc, XMLCRunnerDoc runnerDoc) {
        getNodeInfo(doc, runnerDoc);
    }

    /**
     * Append URLs from an element.
     */
    private void getElementURLs(Element element, XMLCRunnerDoc runnerDoc) {
        String[] attrs = runnerDoc.getElementURLAttrs(element);
        if (attrs == null) {
            return;  // No URLs in this node.
        }
        for (int idx = 0; idx < attrs.length; idx++) {
            String url = element.getAttribute(attrs[idx]);
            if ((url != null) && (url.length() > 0)) {
                urls.addElement(url);
            }
        }
    }

    /**
     * Recursively scan nodes nodes and accumulate information.
     */
    private void getNodeInfo(Node node, XMLCRunnerDoc runnerDoc) {
        if (node instanceof Element) {
            Element elem = (Element)node;
            getElementURLs(elem, runnerDoc);
            String id = runnerDoc.getElementId(elem);
            if ((id != null) && (id.length() > 0)) {
                ids.addElement(id + " => "
                               + runnerDoc.nodeClassToInterface(node));
            }
        }

        for (Node child = node.getFirstChild(); child != null;
             child = child.getNextSibling()) {
            getNodeInfo(child, runnerDoc);
        }
    }

    /**
     * Print document information.
     */
    protected void printInfo(PrintWriter out) {
        int idx;

        out.println("Element IDs:");
        for (idx = 0; idx < ids.size(); idx++) {
            out.println("    " + ids.elementAt(idx));
        }
        out.println("Document URLs:");
        for (idx = 0; idx < urls.size(); idx++) {
            out.println("    " + urls.elementAt(idx));
        }
    }
  }

}

class XMLCRunnerDoc {
  private final String DOC_23 = "org.enhydra.xml.xmlc.dom.XMLCDom";
  private final String DOC_30 = "org.enhydra.xml.xmlc.dom.XMLCDocument";
  private Class  xmlcClass = null;
  private Object xmlcDoc   = null;

  public XMLCRunnerDoc() throws ClassNotFoundException {
     initClass();
  }
  protected Object getXMLCDoc() {
    return xmlcDoc;
  }

  protected void setXMLCDoc(Object d) {
    xmlcDoc = d;
  }

  protected String[] getElementURLAttrs(Element element) {
    String[] attrs = null;
    Class[]  params = new Class[1];
    Object[] args   = new Object[1];
    Method m;
    params[0] = element.getClass();
    args[0]   = element;
    try {
      m = xmlcClass.getMethod("getElementURLAttrs", params);
      attrs = (String[]) m.invoke(xmlcDoc, args);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return attrs;
  }

  protected String getElementId(Element element) {
    String   id     = null;
    Class[]  params = new Class[1];
    Object[] args   = new Object[1];
    Method m;
    params[0] = element.getClass();
    args[0]   = element;
    try {
      m  = xmlcClass.getMethod("getElementId", params);
      id = (String) m.invoke(xmlcDoc, args);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return id;
  }

  protected String nodeClassToInterface(Node node) {
    String   iName  = null;
    Class[]  params = new Class[1];
    Object[] args   = new Object[1];
    Method m;
    params[0] = node.getClass();
    args[0]   = node;
    try {
      m  = xmlcClass.getMethod("nodeClassToInterface", params);
      iName = (String) m.invoke(xmlcDoc, args);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return iName;
  }


  protected String toDocument() {
    String   doc    = null;
    Class[]  params = new Class[0];
    Object[] args   = new Object[0];
    Method m;
    try {
      m = xmlcClass.getMethod("toDocument", params);
      doc = (String) m.invoke(xmlcDoc, args);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return doc;

  }

  protected Document getDocument() {
    Document doc    = null;
    Class[]  params = new Class[0];
    Object[] args   = new Object[0];
    Method m;
    try {
      m = xmlcClass.getMethod("getDocument", params);
      doc = (Document) m.invoke(xmlcDoc, args);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return doc;
  }

  private void initClass() throws ClassNotFoundException {
     try {
       xmlcClass = Class.forName(DOC_30);
     } catch (ClassNotFoundException e) {
       xmlcClass = Class.forName(DOC_23);
     }
  }

}


/*--- formatting done in "Sun Java Convention" style on 01-16-2000 ---*/

